
# q-expansion of newform 1088.4.a.n, downloaded from the LMFDB on 06 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1088
weight = 4
poly_data = [-3, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [-2, 1], [6, -2], [-18, 3], [10, -13], [42, 12], [17, 0], [-16, 38], [22, -25], [198, 26], [58, 37], [70, -90], [-30, -4], [-320, 18], [-248, 60], [-118, -28], [-288, -86], [174, -146], [-764, -16], [-438, -151], [-190, -44], [86, -173], [512, -226], [-422, 332], [18, 56], [242, 180], [-24, -216], [534, 481], [-258, 86], [-990, 48], [-2092, -6], [710, 61], [-406, 316], [-930, -347], [1314, 344], [-988, -82], [-982, 232], [-3550, 11], [-1046, 177], [-1098, -358], [312, 202], [-850, 818], [424, 500], [-814, 800], [-194, -470], [-1486, -723], [-4430, -445], [2748, -258], [-3590, 471], [-390, 1096], [-2270, 332], [-3648, -344], [-1550, -1304], [4308, -388], [1138, 756], [-3172, -494], [5270, -286], [3056, 960], [6310, 350], [-4038, -728], [4714, -853], [-2830, -72], [-3324, -1128], [-4350, -1211], [-5822, 4], [-3554, 2030], [4912, -630], [-2494, -264], [162, -449], [3314, -844], [1698, -2368], [-2460, 1462], [10526, 103], [-5566, 2468], [6854, -1047], [8292, -414], [-12054, 336], [-4194, 414], [5426, -1000], [4490, 1088], [4858, -3305], [7010, -1316], [166, 3435], [-8222, 196], [-6886, 569], [556, 4008], [-9070, -184], [5562, -308], [4210, -900], [-392, 2484], [-1656, 3026], [-7622, -2675], [1782, 1023], [8136, 3062], [8570, -2313], [14478, -1325], [538, -2528], [-4030, -2580], [-572, -3412], [12230, 2378], [302, -783], [12170, -1156], [-6544, -3802], [4730, 600], [-9662, -577], [-19822, 668], [-22272, 786], [17362, 1552], [-368, -956], [-782, 1044], [13794, -1751], [-926, 1344], [130, 2892], [7714, 4983], [-5388, -1498], [5426, -2936], [-15890, 713], [3528, 7864], [3934, 7726], [11756, -1508], [-5014, -5788], [-22894, -1720], [-11898, -1554], [9022, 797], [-10534, 647], [3930, -580], [-6106, -8122], [1266, -927], [-17536, -2788], [-13342, -580], [-5000, 4338], [-13118, -1235], [2410, 253], [8554, 7224], [6298, 7196], [5650, 3388], [1146, 2488], [-27802, 149], [-5566, 6924], [-2846, -9052], [12518, -4943], [1366, 2942], [-1490, 5283], [-11558, 7907], [-31638, 920], [-18666, -457], [11318, -7986], [4450, 3204], [9744, 2338], [26808, 732], [30302, -5994], [-22782, -3608], [6964, 448], [-14214, -7367], [-17574, -1285], [-9970, 4087], [-9960, 184], [20594, 1280], [24746, 3432], [43646, 3638], [-2570, -12243], [-15974, 3068], [-8396, -7042], [12632, -2690], [-34766, -2832], [23322, -3431], [22834, -7159], [-1618, 1298], [-14126, -6752], [8798, 5482], [19100, 9216], [-25750, -9196], [4758, 10439], [426, -3092], [-22100, 10558], [19618, 9932], [22910, -10923], [1694, -6350], [21068, 6042], [-23038, 2696], [-25216, -1232], [-6562, -11415], [-32330, -530], [1770, -8248], [23292, 4870], [-12430, 8620], [11574, -10670], [29716, -4616], [7682, -6780], [5506, -11367], [27362, 7840], [24594, -5345], [12032, 8334], [-18494, -592], [22110, 12065], [-19198, 7268], [32674, 12512], [20830, -5874], [-57022, -3456], [-10632, 9568], [25782, -10470], [-14786, 1543], [-5630, -7928], [46738, 10004], [-7652, 3464], [-13326, -7624], [-14424, 10804], [-50248, 2658], [-15966, 2692], [-25332, 2528], [38418, 11152], [33130, -7672], [-58570, 847], [32088, -2714], [-33682, -14717], [-7086, -10844], [-40544, 8248], [-35086, 13168], [13278, -7869], [-20070, -16500], [23186, -10328], [48354, 7797], [15330, 4880], [51154, -5863], [51732, 14016], [35170, 2004], [8514, -24228], [12966, 16419], [-21916, -8730], [-15654, 467], [-15070, 15716], [29166, 11457], [6884, 14290], [-23718, 18780], [63412, 8836], [4964, 8778], [90962, 4696], [4710, -10786], [-31314, -3899], [76692, 3134], [-30606, -101], [-12852, -15672], [-10832, -16148], [9538, 6008], [-11646, -8656], [13970, 11597], [52330, -17155], [19906, 25051], [8, -9418], [12460, -1426], [-77734, 12864], [-37310, 56], [24108, 2106], [-69550, 300], [12098, 17272], [-59652, -1676], [28398, 24074], [32182, 27521], [79286, 6614], [-70166, -340], [-11318, -27059], [39828, 1784], [-44954, -1426], [-50434, -7514], [5362, -296], [-16124, -336], [-32414, -9484], [67786, -10720], [-76502, -15325], [-41598, -2176], [638, 5246], [4716, -13460], [3098, -35608], [94484, -5894], [60786, 16480], [26116, -114], [-41304, -354], [32090, -19252], [43658, -276], [84600, 13050], [30248, 18308], [7474, 40629], [-51946, 23871], [-13430, -548], [64190, -7867], [-28992, 6600], [-27470, -1456], [-31458, 14650], [11740, 19818], [115506, -1564], [-58890, 14586], [-47778, -2695], [-26742, -4580], [30882, 36639], [13046, 1530], [-124946, -7946], [-101896, -692], [-92766, -1932], [101698, -17313], [33328, -562], [44266, -15588], [118962, 12304], [1790, -18865], [-1474, -1575], [-21434, 10665], [-37934, -30504], [-1740, -22956], [41150, -25794], [-5368, -14336], [138066, 600], [-123226, -14965], [17094, 37094], [-56526, -15944], [-23224, 15530], [-111024, -10292], [-92950, 10248], [34600, 32162], [5882, -38459], [-10654, -27528], [-8334, -46672], [49370, -5553], [-132302, -9796], [-36806, -26400], [-6640, 53800], [-56862, 24428], [20882, 41776], [-144738, 921], [27522, 12727], [22650, -20691], [36550, -22226], [61966, -21834], [-102258, 13694], [12802, 13201], [-36220, 7984], [69362, 12287], [-23670, -25893], [6674, -6172], [-40062, 12184], [59554, -25828], [3604, -32862], [108842, 16676], [-42406, -2744], [78246, 11606], [147416, 15136], [-55014, -16868], [-113838, 1331], [-18826, -8538], [-112420, -15080], [-37038, -3047], [23726, -11038], [-117992, 15826], [-51054, 32732], [87306, 10320], [32282, 22205], [-10134, 29352], [-164998, 7084], [-14244, -21570], [100958, 15405], [-92206, -15896], [12124, -29286], [112046, 33498], [39826, -29276], [64272, 4896], [-146550, 8275], [-221632, 3894], [-25910, 32536], [124230, -4606], [-39296, 43012], [108434, -2876], [108688, -15122], [-141734, -20261], [-47282, -53681], [136418, 2468], [97304, -45800], [-146914, 16486], [-105466, 35661], [1086, -21161], [93714, -29052], [-27758, -44416], [30378, -32160], [16598, 24378], [161898, -2392], [50802, -45691], [-6174, 38360], [42178, -1973], [-25098, -22137], [-69140, 27182], [-85302, -35224], [208990, -12907], [-68976, 51464], [171394, 9464], [-48834, 43082], [44708, 17660], [-42148, 50836], [52588, 30914], [8194, 28052], [30752, -63440], [-111766, 3576], [121306, -20677], [113874, -32900], [58646, -23534], [-20334, 2892], [-29496, 7308], [52162, -44092], [-190302, 9120], [50218, -24591], [50546, -19520], [-134366, 27964], [162848, -19970], [35326, 12913], [-98718, -12936], [79930, -48496], [61412, -36524], [178374, -34019], [86010, 51344], [147702, 37274], [-111402, -40965], [161690, -6119], [-22222, -16912], [88672, 59564], [18306, -9940], [-65618, -70774], [-45558, -43443], [164280, 1014], [153778, -19836], [146826, 7664], [-123258, -3099], [244018, -1180], [-29420, 31900], [131470, 15019]]
